<?php

namespace Drupal\danse\Form;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\danse\Service;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Let users to subscribe to available events.
 */
class Subscriptions extends FormBase {

  /**
   * @var \Drupal\danse\Service
   */
  protected $danse;

  /**
   * Subscriptions constructor.
   *
   * @param \Drupal\danse\Service $danse
   */
  public function __construct(Service $danse) {
    $this->danse = $danse;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('danse.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'danse_user_subscriptions';
  }

  /**
   * Checks access for the DANSE subscription form.
   *
   * @param \Drupal\user\UserInterface|null $user
   *   (optional) The owner of the shortcut set.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function checkAccess(UserInterface $user = NULL) {
    if ($user->id() === $this->currentUser()->id() && $user->isAuthenticated() && $this->configFactory()->get('danse.settings')->get('subscriptions_as_tab')) {
      $form = [];
      $this->danse->buildUserSubscriptionForm($form, $user, FALSE);
      if (isset($form['danse'])) {
        return AccessResult::allowed();
      }
    }
    return AccessResult::forbidden();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, UserInterface $user = NULL) {
    $this->danse->buildUserSubscriptionForm($form, $user, FALSE);
    if (isset($form['danse'])) {
      $form['account'] = ['#type' => 'value', '#value' => $user];
      $form['actions'] = ['#type' => 'actions'];
      $form['actions']['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Save subscriptions'),
        '#button_type' => 'primary',
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->danse->saveUserSubscriptionForm($form_state, $form_state->getValue('account'));
  }

}
